<?php

namespace Drp\NativeEventApp;

use Closure;
use GraphQL\Error\Error;
use Illuminate\Http\Response;
use Nuwave\Lighthouse\Exceptions\RendersErrorsExtensions;
use Nuwave\Lighthouse\Execution\ErrorHandler;

class AddErrorCodes implements ErrorHandler
{
    const GRAPHQL_ERROR = 'graphql';

    public static function handle(Error $error, Closure $next): array
    {
        $underlyingException = $error->getPrevious();

        throw_if(config('native-event-app.debug.disable_error_codes'), $underlyingException);

        $error = new Error(
            $error->getMessage(),
            $error->getNodes(),
            $error->getSource(),
            $error->getPositions(),
            $error->getPath(),
            $underlyingException,
            self::extension($error, $underlyingException)
        );

        return $next($error);
    }

    private static function extension(Error $error, ?\Throwable $underlyingException): array
    {
        $extension = ['code' => self::getCode($error, $underlyingException)];

        if ($underlyingException instanceof RendersErrorsExtensions) {
            $extension = array_merge($extension, $underlyingException->extensionsContent());
        }

        return $extension;
    }

    private static function getCode(Error $error, ?\Throwable $underlyingException): string
    {
        if ($underlyingException) {
            return $underlyingException->getCode();
        }

        if ($error->getCategory() === self::GRAPHQL_ERROR) {
            return Response::HTTP_BAD_REQUEST;
        }

        return Response::HTTP_INTERNAL_SERVER_ERROR;
    }
}
