<?php

namespace Drp\NativeEventApp\Auth;

use Illuminate\Support\Str;
use Laravel\Passport\Bridge\ClientRepository;
use Laravel\Passport\Bridge\Scope;
use Laravel\Passport\Bridge\User;
use League\OAuth2\Server\AuthorizationServer;
use League\OAuth2\Server\Entities\ClientEntityInterface;
use League\OAuth2\Server\RequestTypes\AuthorizationRequest;
use Zend\Diactoros\Response as Psr7Response;

class AuthTokenGenerator
{
    /** @var ClientRepository */
    private $clientRepository;

    /** @var AuthorizationServer */
    private $authorizationServer;

    public function __construct(ClientRepository $clientRepository, AuthorizationServer $authorizationServer)
    {
        $this->clientRepository = $clientRepository;
        $this->authorizationServer = $authorizationServer;
    }

    public function generate(string $authIdentifier): string
    {
        $authRequest = $this->authRequest($authIdentifier);

        return $this->sendAuthRequest($authRequest);
    }

    public function sendAuthRequest(AuthorizationRequest $authRequest): string
    {
        return $this
            ->authorizationServer
            ->completeAuthorizationRequest($authRequest, new Psr7Response)
            ->getBody();
    }

    private function authRequest(string $authIdentifier): AuthorizationRequest
    {
        return tap(new AuthorizationRequest, function (AuthorizationRequest $authRequest) use ($authIdentifier): void {
            session()->put('state', $state = Str::random(40));
            $authRequest->setClient($this->getClient());
            $authRequest->setUser(new User($authIdentifier));
            $authRequest->setGrantTypeId('authorization_token');
            $authRequest->setAuthorizationApproved(true);
            $authRequest->setState($state);
            $authRequest->setScopes([new Scope('all')]);
        });
    }

    private function getClient(): ClientEntityInterface
    {
        return $this->clientRepository->getClientEntity(
            config('native-event-app.auth.client_id'),
            'authorization_token',
            config('native-event-app.auth.client_secret')
        );
    }
}
