<?php

namespace Drp\NativeEventApp\Test\Api\Auth;

use Drp\NativeEventApp\Models\Device;
use Drp\NativeEventApp\Test\Api\GraphQLTestCase;
use Laravel\Passport\Token;
use League\OAuth2\Server\AuthorizationServer;
use Mockery;
use Spatie\Snapshots\MatchesSnapshots;
use Zend\Diactoros\Response as Psr7Response;
use Zend\Diactoros\Response\JsonResponse;
use Zend\Diactoros\ServerRequest;

class RefreshTokenTest extends GraphQLTestCase
{
    use MatchesSnapshots;

    /** @test */
    public function throws_renderable_exception(): void
    {
        $query = 'mutation {
              refreshToken(refreshToken:"1234", deviceId:"fake-device-id") {
                accessToken
              }
            }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function can_exchange_refresh_code_for_token(): void
    {
        $device = factory(Device::class)->create(['id' => '9876', 'access_token_id' => 22]);
        $refreshToken = 'ABC.ABC.ABC';
        $newAccessToken = $this->prepareToken();

        $this->mock(AuthorizationServer::class, function ($mock) use ($newAccessToken): void {
            $mock->shouldReceive('respondToAccessTokenRequest')
                ->once()
                ->with(Mockery::on(function (ServerRequest $request) {
                    return $request->getParsedBody() === $this->expectedRequestBody();
                }), Mockery::type(Psr7Response::class))
                ->andReturn($this->fakeOAuthTokenResponse($newAccessToken));
        });

        $query = "mutation {
              refreshToken(refreshToken:\"$refreshToken\", deviceId: \"$device->id\") {
                accessToken
                refreshToken
                expiresIn
                tokenType
              }
            }";

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));

        $this->assertEquals(1, $device->fresh()->access_token_id);
    }

    private function prepareToken(): string
    {
        $token = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJub25lIiwianRpIjoiMSJ9.eyJqdGkiOiIxIiwiZXhwIjoxNTg3MzY5NjIyfQ.';
        Token::create([
            'id' => 1,
            'client_id' => config('native-event-app.auth.client_id'),
            'revoked' => 0,
        ]);

        return $token;
    }

    private function expectedRequestBody(): array
    {
        return [
            'refresh_token' => 'ABC.ABC.ABC',
            'device_id' => '9876',
            'client_id' => '1234',
            'client_secret' => '1234',
            'grant_type' => 'refresh_token',
        ];
    }

    protected function fakeOAuthTokenResponse(string $accessToken): Psr7Response
    {
        return new JsonResponse([
            'token_type' => 'Bearer',
            'expires_in' => '12345',
            'access_token' => $accessToken,
            'refresh_token' => 'REFRESH TOKEN',
        ]);
    }
}
