<?php

namespace Drp\NativeEventApp\Test\Unit\FileUploads;

use Drp\NativeEventApp\FileUploads\Facades\FileUpload as FileUploadFacade;
use Drp\NativeEventApp\FileUploads\LocalDriver;
use Drp\NativeEventApp\Models\FileUpload;
use Drp\NativeEventApp\Test\TestCase;
use Illuminate\Http\Testing\File;
use Illuminate\Support\Facades\Storage;

class LocalDriverTest extends TestCase
{
    use FileUploadContractTest;

    public function driver(): LocalDriver
    {
        config()->set('native-event-app.files.local.disk', 'local');
        config()->set('native-event-app.files.local.path', 'uploads');

        Storage::fake('local');

        return FileUploadFacade::createLocalDriver();
    }

    public function storeFile(string $path, string $uuid): void
    {
        $file = new File($uuid . '.png', fopen($path, 'rw+'));
        $file->storeAs(
            config('native-event-app.files.local.path') . '/' . $uuid,
            $uuid . '.png',
            'local'
        );
    }

    public function assertFileCorrectlyUploaded(FileUpload $fileUpload, string $uuid): void
    {
        $this->assertEquals('local', $fileUpload->disk);
        $this->assertEquals('uploads/' . $uuid, $fileUpload->path);

        Storage::disk(config('native-event-app.files.local.disk'))->assertExists($fileUpload->path);
    }
}
