<?php

namespace Drp\NativeEventApp\Test\Unit\FileUploads;

use Drp\NativeEventApp\FileUploads\Facades\FileUpload as FileUploadFacade;
use Drp\NativeEventApp\FileUploads\S3Driver;
use Drp\NativeEventApp\Models\FileUpload;
use Drp\NativeEventApp\Test\TestCase;
use GuzzleHttp\Client;

class S3DriverTest extends TestCase
{
    use FileUploadContractTest;

    protected function setUp(): void
    {
        $this->markTestSkipped('Live Amazon S3 Test: Remove setUp and add credentials to test');
    }

    public function driver(): S3Driver
    {
        config()->set('native-event-app.files.s3.key', 'ENTER_ACCESS_KEY_ID');
        config()->set('native-event-app.files.s3.secret', 'ENTER_ACCESS_SECRET');
        config()->set('native-event-app.files.s3.bucket', 'ENTER_BUCKET_NAME');

        return FileUploadFacade::createS3Driver();
    }

    public function storeFile(string $path, string $uuid): void
    {
        $url = $this->driver()->createUrl($uuid, '+1 minute');

        $client = new Client();
        $client->request('PUT', $url['url'], [
            'multipart' => [
                [
                    'name'     => 'test.png',
                    'contents' => fopen($path, 'r'),
                ],
            ],
        ]);
    }

    public function assertFileCorrectlyUploaded(FileUpload $fileUpload, string $uuid): void
    {
        $this->assertEquals(config('native-event-app.files.s3.bucket'), $fileUpload->disk);
        $this->assertEquals($uuid, $fileUpload->path);

        $this->assertTrue($this->driver()->getStorageClient()->doesObjectExist(
            config('native-event-app.files.s3.bucket'),
            $uuid
        ));
    }
}
