<?php

namespace Drp\NativeEventApp\Test\Unit\Modules;

use Drp\NativeEventApp\Exceptions\InvalidModules;
use Drp\NativeEventApp\Modules\Modules;
use Drp\NativeEventApp\Modules\Tags;
use Drp\NativeEventApp\Test\Support\Fakes\Modules\FakeModule;
use Drp\NativeEventApp\Test\Support\InteractsWithModules;
use Drp\NativeEventApp\Test\TestCase;

class ModulesTest extends TestCase
{
    use InteractsWithModules;

    /**
     * @test
     */
    public function cant_add_modules_to_modules()
    {
        $this->expectException(InvalidModules::class);

        Modules::make([
            Modules::make([])
        ]);
    }

    /**
     * @test
     */
    public function can_only_add_a_module_to_modules()
    {
        $this->expectException(InvalidModules::class);

        Modules::make([
            'OTHER'
        ]);
    }

    /**
     * @test
     */
    public function can_be_instantiated()
    {
        $modules = Modules::make([
            FakeModule::make('header')->text('CONTENT'),
            FakeModule::make('paragraph')->text('CONTENT 2')->other('OTHER CONTENT'),
        ]);

        $this->assertEquals(2, $modules->count());
        $this->assertEquals('CONTENT', $modules->first()->getText());
        $this->assertEquals('CONTENT 2', $modules->last()->getText());
        $this->assertEquals('OTHER CONTENT', $modules->last()->getOther());
    }

    /**
     * @test
     */
    public function can_be_instantiated_from_json()
    {
        $data = json_encode([
            [
                'type' => 'fake_module',
                'text' => 'CONTENT'
            ],
            [
                'type' => 'fake_module',
                'text' => 'CONTENT 2',
                'other' => 'OTHER CONTENT'
            ]
        ]);

        $modules = Modules::fromString($data);

        $this->assertEquals(2, $modules->count());
        $this->assertEquals('CONTENT', $modules->first()->text);
        $this->assertEquals('CONTENT 2', $modules->last()->text);
        $this->assertEquals('OTHER CONTENT', $modules->last()->other);
    }
}
